//********************************************************************************************************************************
//*                                                                                                                              *
//*     3delite's Helper functions 1.5  3delite 2006-2022                                                                       *
//*                                                                                                                              *
//* Win32, Win64, OSX32, iOS, Android compatible.                                                                                *
//*                                                                                                                              *
//* 3delite's Helper functions is free (a mention in the credits would be cool if you use it!).                                  *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//* If you have any of the aforementioned please email: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//********************************************************************************************************************************

unit Helper3delite;

interface

Uses
    SysUtils;

type
    TMouseButtonType = (mbtLeft, mbtMiddle, mbtRight);

    function IntToStrDigits(Value: Int64; Digits: Integer): String;
    function IncludeTrailingPathDelimiterHTTP(const S: string): string;
    function URLEncode(const S: String; const InQueryString: Boolean): String;
    function CreateUnicodeFile(FileName: String): Boolean;
    function IsNewerVersion(CurrentVersion, NewVersion: String): Boolean;
    function IntToStrW(const Value: Int64; const Digits: Word): String;
    function MatchFileType(const FileName: String; Extensions: String): Boolean;
    function ExtractFileNameWithoutExt(const FileName: String): String;
    function ExtractURL(const URLFileName: string): String;
    function LastPos(SubStr: Char; S: string): Integer; overload;
    function LastPosLong(SubStr, S: string): Integer; overload;
    function LastPosEx(SubStr, S: string): Integer; overload;
    function LastPos(SubStr: array of Char; S: string): Integer; overload;
    function PToStr(Address: Pointer): String;
    function GetLogFileName(FileNameStart, Folder: String): String;
    function MIMEDecodeStr(MIMEStr: String): String;
    function GetHTMLTitle(FileName: String): String;
    function CreateDumbFile(FileName: String): Boolean;
    function CreateDumbFileEx(FileName: String; Size: Integer): Boolean;
    function StrConvertToFileName(StrFileName: String): String;
    function StrConvertToSQLName(Text: String): String;
    function HexToInt(HexStr: String): Integer;
    function MakeInt64(LowDWord, HiDWord: Cardinal): Int64;
    function StrXOR(Text: String; Key: Byte): String;
    function BitSet(Value: Cardinal; BitNo: Byte; On: Boolean): Cardinal;
    function WriteText2File(FileName, Text: String): Boolean;
    procedure LogStrings(FileName, Text: String);
    function BGR2RGB(BGRValue: Cardinal): Cardinal;
    function Bytes2MB(Bytes: Int64): String; overload;
    function Bytes2MB(Bytes: Extended): String; overload;
    function Secs2Time(Secs: Int64): String; overload;
    function Secs2TimeFast(Seconds: Double): String; overload;
    function Secs2Time(Secs: Double): String; overload;
    function mSec2Time(mSec: Int64): String;
    function Sec2Time(Sec: Int64): String;
    function mSec2TimeEx(mSec: Int64; Format: Integer): String;
    function mSec2HoursPart(mSec: Int64): Int64;
    function mSec2MinutesPart(mSec: Int64): Int64;
    function mSec2SecondsPart(mSec: Int64): Int64;
    function mSec2MilliSecondsPart(mSec: Int64): Int64;
    function mSec2TimeSecs(mSec: Int64): String;
    function NGetFileSize(FileName: String): Int64; overload;
    function NGetFileSizeFrom(SearchRec: TSearchRec): Int64; overload;
    function EncodeUnicode(UniString: String): String;
    function DecodeUnicode(UnicodeString: String): String;
    function IntToBin1(Value: Longint; Digits: Integer): string;
    function IntToBin2(d: Longint): string;
    function GetFileModDate(FileName: String): TDateTime;
    function ExtractFileNameFromURL(const FileName: string): string;
    function HTMLDecode(const AStr: String): String;
    function DateTimeDiff(Start, Stop: TDateTime): Int64;
    function DateTimeDiffmSec(Start, Stop: TDateTime): Int64;
    function Time2mSec(Value: TTime): Int64;
    function Linear2dB(Value: Double): String;
    function dB2LinearPercent(Value: Double): String;
    function ReplaceNewLines(Text: String): String;
    function FFTIndex2Frequency(Index: Integer; Length: Integer; Samplerate: Integer): Integer;
    function FFTFrequency2Index(Frequency: Integer; Length: Integer; Samplerate: Integer): Integer;
    function GoUpOneFolder(Folder: String): String;
    function GoUpOneCastFolder(Folder: String): String;

    function Min(const B1, B2: Integer): Integer; inline;
    function Max(const B1, B2: Integer): Integer; inline;

    function ExtractSeasonEpisode(Text: String; var S: Integer; var E: Integer): Boolean;

implementation

Uses
    Classes,
    StrUtils,
    Math,
    System.Character;

function CreateUnicodeFile(FileName: String): Boolean;
var
    UnicodeStringList: TStrings;
    Enc: TEncoding;
begin
    Result := False;
    if FileExists(FileName) then begin
        Exit;
    end;
    UnicodeStringList := tStringList.Create;
    UnicodeStringList.Clear;
    Enc := TUnicodeEncoding.Create;
    try
        UnicodeStringList.SaveToFile(FileName, Enc.Unicode);
        Result := True;
    finally
        Enc.Free;
        UnicodeStringList.Free;
    end;
end;

procedure Split(const Delimiter: Char; Input: string; const Strings: TStrings);
begin
    Strings.Clear;
    Strings.Delimiter := Delimiter;
    Strings.DelimitedText := Input;
end;

function IsNewerVersion(CurrentVersion, NewVersion: String): Boolean;
var
  CurrentMajorVersion, CurrentMinorVersion, CurrentReleaseVersion, CurrentBuildVersion: Integer;
  NewMajorVersion, NewMinorVersion, NewReleaseVersion, NewBuildVersion: Integer;
  Exploded: TStringList;
begin

    CurrentMajorVersion := 0;
    CurrentMinorVersion := 0;
    CurrentReleaseVersion := 0;
    CurrentBuildVersion := 0;
    NewMajorVersion := 0;
    NewMinorVersion := 0;
    NewReleaseVersion := 0;
    NewBuildVersion := 0;

    Exploded := TStringList.Create;
    try
        Split('.', CurrentVersion, Exploded);
        if Exploded.Count > 0 then begin
            CurrentMajorVersion := StrToIntDef(Exploded[0], 0);
        end;
        if Exploded.Count > 1 then begin
            CurrentMinorVersion := StrToIntDef(Exploded[1], 0);
        end;
        if Exploded.Count > 2 then begin
            CurrentReleaseVersion := StrToIntDef(Exploded[2], 0);
        end;
        if Exploded.Count > 3 then begin
            CurrentBuildVersion := StrToIntDef(Exploded[3], 0);
        end;
    finally
        Exploded.Free;
    end;

    Exploded := TStringList.Create;
    try
        Split('.', NewVersion, Exploded);
        if Exploded.Count > 0 then begin
            NewMajorVersion := StrToIntDef(Exploded[0], 0);
        end;
        if Exploded.Count > 1 then begin
            NewMinorVersion := StrToIntDef(Exploded[1], 0);
        end;
        if Exploded.Count > 2 then begin
            NewReleaseVersion := StrToIntDef(Exploded[2], 0);
        end;
        if Exploded.Count > 3 then begin
            NewBuildVersion := StrToIntDef(Exploded[3], 0);
        end;
    finally
        Exploded.Free;
    end;

    try
        if NewMajorVersion = CurrentMajorVersion then begin
            if NewMinorVersion = CurrentMinorVersion then begin
                if NewReleaseVersion = CurrentReleaseVersion then begin
                    if NewBuildVersion = CurrentBuildVersion then begin
                        Result := False;
                        Exit;
                    end;
                end;
            end;
        end;

        if NewMajorVersion > CurrentMajorVersion then begin
            Result := True;
            Exit;
        end else begin
            if NewMinorVersion > CurrentMinorVersion then begin
                Result := True;
                Exit;
            end else begin
                if NewReleaseVersion > CurrentReleaseVersion then begin
                    Result := True;
                    Exit;
                end else begin
                    if NewBuildVersion > CurrentBuildVersion then begin
                        Result := True;
                        Exit;
                    end;
                end;
            end;
        end;

        Result := False;

    except
        Result := False;
    end;
end;

function IntToStrW(const Value: Int64; const Digits: Word): String;
var
  Res: String;
begin
    try
        Res := IntToStr(Value);
        while Length(Res) < Digits
            do Res := '0' + Res;
        Result := Res;
    except
        //*
    end;
end;

//* Extensions expects #13#10 delimited list of file extensions without the dot
function MatchFileType(const FileName: String; Extensions: String): Boolean;
var
  Fext: String;
  Exts: TStrings;
  i: Integer;
begin
    Result := False;
    try
        try
            Fext := ExtractFileExt(FileName);
            Fext := UpperCase(Copy(Fext, 2, Length(Fext)));
            Exts := TStringList.Create;
            Exts.Text := UpperCase(Extensions);
            for i := 0 to Exts.Count - 1 do begin
                if Exts[i] = Fext then begin
                    Result := True;
                    Break;
                end;
            end;
        except
            //*
        end;
    finally
        if Assigned(Exts)
            then FreeAndNil(Exts);
    end;
end;

function ExtractFileNameWithoutExt(const FileName: String): String;
var
  I: Integer;
  PlainFileName: String;
begin
    PlainFileName := ExtractFileName(FileName);
    I := LastDelimiter('.' + PathDelim + DriveDelim, PlainFileName);
    if (I = 0)
    OR (PlainFileName[I] <> '.')
        then I := MaxInt;
    Result := Copy(PlainFileName, 1, I - 1);
end;

function ExtractURL(const URLFileName: string): String;
var
  URLFile: TStrings;
begin
    Result := '';
    if NOT FileExists(URLFileName)
        then Exit;
    URLFile := TStringList.Create;
    try
        try
            URLFile.LoadFromFile(URLFileName);
            if Copy(URLFile[3], 1, 3) = 'URL'
                then Result := Copy(URLFile[3], 5, Length(URLFile[3]));
        except
            //*
        end;
    finally
        URLFile.Free;
    end;
end;

{
SIZE s;
HDC hdc = CreateCompatibleDC(NULL);
SelectObject( hdc, 'ide jn a HFONT' );
GetTextExtentPoint32( hdc, "blabla", 6, &s );
ReleaseDC( NULL, hdc );
}
{
function Str2Real(S: String): Real;
var
  Round1, Fract1, Sint: String;
  RNum, FNum: ShortInt;
  RealFract: Real;
  // eljel jelzs
  Sign: Boolean;
begin
    Sign := False;
    if S[1]= '-'
        then Sign := True;
    Round1 := StrPick(S, '.', False);
    Fract1 := StrPick(S, '.', True);
    // ha nincs tizedes pont
    if Round1 = '' then begin
        // ha mnusz eljelu
        if Sign = True then begin
            Sint := '-' + Fract1 + '.0';
            Round1 := StrPick(Sint, '.', False);
            Fract1 :='0';
        end else begin
            Sint := Fract1 + '.0';
            Round1 := StrPick(Sint, '.', False);
            Fract1 := '0';
        end;
    end;
    RNum:= StrToInt(Round1);
    FNum:= StrToInt(Fract1);
    RealFract := FNum / 10;
    if RNum > 0
        then Result := RNum + RealFract;
    if RNum < 0 then begin
        if FNum > 0
            // csak 0.5 vgzdsre!
            then Result := (RNum - 1) + RealFract
            else Result:= RNum;
        end;
    if RNum = 0 then begin
        if Round1[1] = '-'
            then Result := (RNum - 1) + RealFract
            else Result := RNum + RealFract;
    end;
end;
}

function LastPos(SubStr: Char; S: string): Integer; overload;
var
  Found, Pos: integer;
begin
    Found := 0;
    try
        try
            Pos := Length(S);
            while (Pos > 0)
            AND (Found = 0)
            do begin
                if Copy(S, Pos, 1) = SubStr
                    then Found := Pos;
                Dec(Pos);
            end;
        except
            //*
        end;
    finally
        Result := Found;
    end;
end;

function LastPosLong(SubStr, S: string): Integer; overload;
var
  Found, Len, Pos: integer;
begin
    //Result := 0;
    Found := 0;
    try
        try
            Pos := Length(S);
            Len := Length(SubStr);
            while (Pos > 0)
            AND (Found = 0)
            do begin
                if Copy(S, Pos, Len) = SubStr then begin
                    Found := Pos;
                    Exit;
                end;
                Dec(Pos);
            end;
        except
            //*
        end;
    finally
        Result := Found;
    end;
end;

function LastPosEx(SubStr, S: string): Integer; overload;
var
  Found, Pos, i: Integer;
  //Len: Integer;
begin
    //Result := 0;
    Found := 0;
    try
        try
            Pos := Length(S);
            //Len := Length(SubStr);
            while (Pos > 0)
            AND (Found = 0)
            do begin
                for i := 1 to Length(SubStr) do begin
                    if Copy(S, Pos, 1) = SubStr[i] then begin
                        Found := Pos;
                        Exit;
                    end;
                end;
                Dec(Pos);
            end;
        except
            //*
        end;
    finally
        Result := Found;
    end;
end;

function LastPos(SubStr: array of Char; S: string): Integer; overload;
var
  Found, Pos, i: Integer;
  //Len: Integer;
begin
    //Result := 0;
    Found := 0;
    try
        try
            Pos := Length(S);
            //Len := Length(SubStr);
            while (Pos > 0)
            AND (Found = 0)
            do begin
                for i := 0 to Length(SubStr) do begin
                    if Copy(S, Pos, 1) = SubStr[i] then begin
                        Found := Pos;
                        Exit;
                    end;
                end;
                Dec(Pos);
            end;
        except
            //*
        end;
    finally
        Result := Found;
    end;
end;

function PToStr(Address: Pointer): String;
begin
    if Address = nil then begin
        Result := 'nil';
        Exit;
    end;
    try
        Result := '$' + IntToHex(NativeUInt(Address), 8);
    except
        Result := 'unknown';
    end;

end;

function GetLogFileName(FileNameStart, Folder: String): String;
var
  LogFileName: String;
begin
    Result := '';
    try
        LogFileName := DateToStr(Now) + TimeToStr(Now);
        LogFileName := FileNameStart + StrConvertToFileName(LogFileName);
        Result := IncludeTrailingPathDelimiter(Folder) + LogFileName + '.log';
    except
        //*
    end;
end;

function MIMEDecodeStr(MIMEStr: String): String;
var
  Buffer: String;
  Botu: Char;
  i: Integer;
  StartPos, Offset: Int64;
begin
    try
        Result := MIMEStr;

        StartPos := Pos('Q?', Result) + 2;

        if StartPos > 2 then begin

            Result := Copy(Result, StartPos, Length(Result));

            Buffer := '';
            Offset := 0;
            for i := 1 to Length(Result) - 2  do begin

                Botu := Result [i + Offset];

                if Botu = '=' then begin
                    Buffer := Buffer + Chr(HexToInt(Result [i + 1 + Offset] + Result [i + 2 + Offset]));
                    Inc(Offset, 2);
                    //Inc(i, 2);
                end else Buffer := Buffer + Result [i + Offset];

                if i + Offset >= Length(Result) - 2
                    then Break;
            end;

            Result := ANSIReplaceStr(Buffer, '_', ' ');

        end;
    except
        //*
    end;
end;


function GetHTMLTitle(FileName: String): String;
var
  Dok: TStrings;
  Fext, Text: String;
  i{, k}: Integer;
  StartPos, EndPos: Int64;
begin

    Result := '';
    if NOT FileExists(FileName)
        then Exit;

    Fext :=  AnsiUpperCase(ExtractFileExt(FileName));

    try
        Dok := TStringList.Create;
        try
            Dok.LoadFromFile(FileName);

            Dok.Text := Copy(Dok.Text, 1, 1024);

            if (Fext = '.HTM')
            OR (Fext = '.HTML')
            then begin
                //* <title>Title</title>
                //* Limit to 20KB
                Text := UpperCase(Copy(Dok.Text, 1, 1024 * 4));
                StartPos := Pos('<TITLE>', Text) + 7;
                EndPos := Pos('</TITLE>', Text);
                if EndPos > 0 then begin
                    Result := Copy(Dok.Text, StartPos, EndPos - StartPos);
                    Result := ReplaceStr(Result, (#13#10), ' ');
                end;
            end;

            if (Fext = '.MHT')
            then begin
                //* Subject: Title
                //* Search is limited to only the first 16 lines
                //k := 1;
                for i := 0 to 15 do begin
                    if i >= Dok.Count
                        then Break;
                    if UpperCase(Copy(Dok[i], 1, 8)) = 'SUBJECT:' then begin
                        Result := MIMEDecodeStr(Trim(Copy(Dok[i], 10, 128)));
                        {
                        while (UpperCase(Copy(Dok[i + k], 1, 5)) <> 'DATE:')
                        AND (i <= Dok.Count)
                        do begin
                            Result := Result + MIMEDecodeStr(Trim(Copy(Dok[i + k], 1, Length(Dok[i + k]))));
                            Inc(k);
                        end;
                        }
                        Break;
                    end;
                end;
            end;

            if Result <> '' then begin
                Result := DecodeUnicode(Result);
                //Result := UTF8Decode(Result);
            //    Result := HTTPDecode(Result);
            //    Result := HTMLDecode(Result);
            end;

        except
            //*
        end;
    finally
        FreeAndNil(Dok);
    end;
end;

function CreateDumbFile(FileName: String): Boolean;
var
    Dumb: TStrings;
begin
    try
        Result := False;
        if FileExists(FileName)
            then Exit;
        Dumb := TStringList.Create;
        try
            Dumb.SaveToFile(FileName);
            Result := True;
        finally
            if Assigned(Dumb)
                then FreeAndNil(Dumb);
        end;
    except
        Result := False;
    end;
end;

function CreateDumbFileEx(FileName: String; Size: Integer): Boolean;
var
    Dumb: TFileStream;
    ZeroByte: Byte;
    i: Integer;
begin
    try
        Result := False;
        if FileExists(FileName)
            then Exit;
        try
            ZeroByte := 0;
            Dumb := TFileStream.Create(FileName, fmCreate);
            for i := 1 to Size do begin
                Dumb.Write(ZeroByte, 1);
            end;
            Result := True;
        finally
            if Assigned(Dumb)
                then FreeAndNil(Dumb);
        end;
    except
        Result := False;
    end;
end;

function StrConvertToFileName(StrFileName: String): String;
begin
	Result := StrFileName;
    Result := StringReplace(Result, '*', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '<', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '>', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '"', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '\', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '/', '_', [rfReplaceAll]);
    Result := StringReplace(Result, ':', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '|', '_', [rfReplaceAll]);
    Result := StringReplace(Result, '?', '_', [rfReplaceAll]);
end;

function StrConvertToSQLName(Text: String): String;
begin
	Result := Text;
    Result := StringReplace(Result, '*', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '<', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '>', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '"', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '\', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '/', '_', [rfReplaceAll]);
    Result := StringReplace(Result, ':', '_', [rfReplaceAll]);
	Result := StringReplace(Result, '|', '_', [rfReplaceAll]);
    Result := StringReplace(Result, '?', '_', [rfReplaceAll]);
    Result := StringReplace(Result, '&', '_', [rfReplaceAll]);
    Result := StringReplace(Result, '=', '_', [rfReplaceAll]);
    Result := StringReplace(Result, ' ', '', [rfReplaceAll]);
end;

function HexToInt(HexStr: String): Integer;
begin
    Result := StrToInt('$' + HexStr);
end;

{
function WB_SaveHTMLCode(WebBrowser: TWebBrowser; const FileName: TFileName): Boolean;
var
  ps: IPersistStreamInit;
  fs: TFileStream;
  sa: IStream;
begin
  ps := WebBrowser.Document as IPersistStreamInit;
  fs := TFileStream.Create(FileName, fmCreate);
  try
    sa := TStreamAdapter.Create(fs, soReference) as IStream;
    Result := Succeeded(ps.Save(sa, True));
  finally
    fs.Free;
  end;
end;
}

function MakeInt64(LowDWORD, HiDWORD: Cardinal): Int64;
begin
    Result := LowDWORD OR (Int64(HiDWORD) SHL 32);
end;

function StrXOR(Text: String; Key: Byte): String;
var
  i: Cardinal;
begin
    Result := '';
    if Length(Text) < 1
        then Exit;
    for i := 1 to Length(Text)
        do Result := Result + Chr(Byte(Ord(Text[i])) XOR Key);
end;

function BitSet(Value: Cardinal; BitNo: Byte; On: Boolean): Cardinal;
var
  bMask: Cardinal;
begin
    if On then begin
        // Bit bellts
        bMask := 1;
        bMask := bMask SHL BitNo;
        Result := Value OR bMask;
    end else begin
        // Bit trls
        bMask := 1;
        bMask := $FFFFFFFF - (bMask SHL BitNo);
        Result := Value AND bMask;
    end;
end;

function WriteText2File(FileName, Text: String): Boolean;
var
  sText: TStrings;
begin
    Result := False;
    sText := TStringList.Create;
    try
        try
            if FileExists(FileName)
                then sText.LoadFromFile(FileName);
            sText.Append(Text);
            sText.SaveToFile(FileName);
            Result := True;
        except
            //*
        end;
    finally
        sText.Free;
    end;
end;

procedure LogStrings(FileName, Text: String);
var
    FileStream: TFileStream;
    LogString: String;
    BOM: Byte;
begin
    try
        if FileExists(FileName) then begin
            FileStream := TFileStream.Create(FileName, fmOpenReadWrite OR fmShareDenyWrite)
        end else begin
            FileStream := TFileStream.Create(FileName, fmCreate OR fmShareDenyWrite);
            BOM := $FF;
            FileStream.Write(BOM, 1);
            BOM := $FE;
            FileStream.Write(BOM, 1);
        end;
        try
            FileStream.Seek(0, soFromEnd);
            LogString := Text + (#13#10);
            FileStream.Write(Pointer(LogString)^, Length(LogString) * 2);
        finally
            FileStream.Free;
        end;
    except
        //*
    end;
end;

function BGR2RGB(BGRValue: Cardinal): Cardinal;
begin
    BGRValue := (BGRValue shl 8) shr 8;
   	Result := (BGRValue shr 16) + (((BGRValue shl 16) shr 24) shl 8) + (BGRValue shl 16);
end;

function Bytes2MB(Bytes: Int64): String;
var
    KB: Extended;
    MB: Extended;
    GB: Extended;
    TB: Extended;
    PB: Extended;
    EB: Extended;
    ZB: Extended;
    YB: Extended;
    KBD: Extended;
    MBD: Extended;
    GBD: Extended;
    TBD: Extended;
    PBD: Extended;
    EBD: Extended;
    ZBD: Extended;
    YBD: Extended;
begin
    {
    if Bytes > 1048576 then begin
        if Bytes > 1073741824
            then Result := FloatToStrF((Bytes / 1073741824), ffFixed, 4, 2) + ' GB'
            else Result := FloatToStrF((Bytes / 1048576), ffFixed, 4, 2) + ' MB';
    end else Result := FloatToStrF((Bytes / 1024), ffFixed, 4, 2) + ' KB';
    if Bytes < 1024
 	    then Result := IntToStr(Bytes) + ' Byte';
    }

    KB := 1024.0 * 1024.0;
    MB := 1024.0 * 1024.0 * 1024.0;
    GB := 1024.0 * 1024.0 * 1024.0 * 1024.0;
    TB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    PB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    EB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    ZB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    YB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;

    KBD := 1024.0;
    MBD := 1024.0 * 1024.0;
    GBD := 1024.0 * 1024.0 * 1024.0;
    TBD := 1024.0 * 1024.0 * 1024.0 * 1024.0;
    PBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    EBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    ZBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    YBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;

    if Bytes < 1024 then begin
        Result := IntToStr(Bytes) + ' Byte';
    end else begin
        if Bytes < KB then begin
            Result := FloatToStrF((Bytes / KBD), ffFixed, 4, 2) + ' KB';
        end else begin
            if Bytes < MB then begin
                Result := FloatToStrF((Bytes / MBD), ffFixed, 4, 2) + ' MB';
            end else begin
                if Bytes < GB then begin
                    Result := FloatToStrF((Bytes / GBD), ffFixed, 4, 2) + ' GB'
                end else begin
                    if Bytes < TB then begin
                        Result := FloatToStrF((Bytes / TBD), ffFixed, 4, 2) + ' TB'
                    end else begin
                        if Bytes < PB then begin
                            Result := FloatToStrF((Bytes / PBD), ffFixed, 4, 2) + ' PB'
                        end else begin
                            if Bytes < EB then begin
                                Result := FloatToStrF((Bytes / EBD), ffFixed, 4, 2) + ' EB'
                            end else begin
                                if Bytes < ZB then begin
                                    Result := FloatToStrF((Bytes / ZBD), ffFixed, 4, 2) + ' ZB'
                                end else begin
                                    if Bytes < YB then begin
                                        Result := FloatToStrF((Bytes / YBD), ffFixed, 4, 2) + ' YB'
                                    end else begin
                                        Result := FloatToStrF((Bytes / YBD), ffFixed, 4, 2) + ' YB'
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;

function Bytes2MB(Bytes: Extended): String;
var
    KB: Extended;
    MB: Extended;
    GB: Extended;
    TB: Extended;
    PB: Extended;
    EB: Extended;
    ZB: Extended;
    YB: Extended;
    KBD: Extended;
    MBD: Extended;
    GBD: Extended;
    TBD: Extended;
    PBD: Extended;
    EBD: Extended;
    ZBD: Extended;
    YBD: Extended;
begin

    KB := 1024.0 * 1024.0;
    MB := 1024.0 * 1024.0 * 1024.0;
    GB := 1024.0 * 1024.0 * 1024.0 * 1024.0;
    TB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    PB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    EB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    ZB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    YB := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;

    KBD := 1024.0;
    MBD := 1024.0 * 1024.0;
    GBD := 1024.0 * 1024.0 * 1024.0;
    TBD := 1024.0 * 1024.0 * 1024.0 * 1024.0;
    PBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    EBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    ZBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;
    YBD := 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0 * 1024.0;

    if Bytes < 1024 then begin
        Result := IntToStr(Round(Bytes)) + ' Byte';
    end else begin
        if Bytes < KB then begin
            Result := FloatToStrF((Bytes / KBD), ffFixed, 4, 2) + ' KB';
        end else begin
            if Bytes < MB then begin
                Result := FloatToStrF((Bytes / MBD), ffFixed, 4, 2) + ' MB';
            end else begin
                if Bytes < GB then begin
                    Result := FloatToStrF((Bytes / GBD), ffFixed, 4, 2) + ' GB'
                end else begin
                    if Bytes < TB then begin
                        Result := FloatToStrF((Bytes / TBD), ffFixed, 4, 2) + ' TB'
                    end else begin
                        if Bytes < PB then begin
                            Result := FloatToStrF((Bytes / PBD), ffFixed, 4, 2) + ' PB'
                        end else begin
                            if Bytes < EB then begin
                                Result := FloatToStrF((Bytes / EBD), ffFixed, 4, 2) + ' EB'
                            end else begin
                                if Bytes < ZB then begin
                                    Result := FloatToStrF((Bytes / ZBD), ffFixed, 4, 2) + ' ZB'
                                end else begin
                                    if Bytes < YB then begin
                                        Result := FloatToStrF((Bytes / YBD), ffFixed, 4, 2) + ' YB'
                                    end else begin
                                        Result := FloatToStrF((Bytes / YBD), ffFixed, 4, 2) + ' YB'
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;

function Secs2Time(Secs: Int64): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
begin
    tHours := IntToStr(Secs div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Secs - (StrToInt(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Secs - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    Result := tHours + ':' + tMinutes + ':' + tSecs;
end;

function Secs2TimeFast(Seconds: Double): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Hours: Double;
  Minutes: Double;
  Secs: Double;
  mSecs: Double;
begin
    Hours := Seconds / 3600;
    tHours := IntToStr(Trunc(Hours));
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    Minutes := (Seconds - (Hours * 3600)) / 60;
    tMinutes := IntToStr(Trunc(Minutes));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    Secs := Seconds - (Hours * 3600 + Minutes * 60);
    tSecs := IntToStr(Trunc(Secs));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    mSecs := (Seconds * 1000) - ((Hours * 3600 + Minutes * 60) + Secs) * 1000;
    tmSecs := IntToStr(Trunc(mSecs));
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    tmSecs := Copy(tmSecs, 1, 2);
    Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
end;

function Secs2Time(Secs: Double): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
begin
    if Secs = 4294967295
        then Secs := 0;
    tHours := IntToStr(Trunc(Secs / 3600));
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(Trunc(((Secs - (StrToInt(tHours) * 3600)) / 60)));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Trunc(Secs - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60)));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(Trunc((Secs * 1000) - ((StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60) + StrToInt(tSecs)) * 1000));
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    tmSecs := Copy(tmSecs, 1, 2);
    Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
end;

{
[ms / (1000 * 60 * 60), // hours
 (ms / (1000 * 60)) mod 60, // minutes
 (ms / 1000) mod 60]); // seconds
}

function mSec2Time(mSec: Int64): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    tHours := IntToStr(Seconds div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Seconds - (StrToInt64(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt64(tHours) * 3600 + StrToInt64(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(mSec - ((StrToInt64(tHours) * 3600 + StrToInt64(tMinutes) * 60) + StrToInt64(tSecs)) * 1000);
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    tmSecs := Copy(tmSecs, 1, 2);
    Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
end;

function Sec2Time(Sec: Int64): String;
var
  tDays: String;
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Seconds: Int64;
  mSec: Int64;
begin
    mSec := Sec * 1000;
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    tDays := IntToStr(Seconds div 86400);
    tHours := IntToStr((Seconds - (StrToInt64(tDays) * 86400)) div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Seconds - (StrToInt64(tDays) * 86400) - (StrToInt64(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt64(tDays) * 86400) - (StrToInt64(tHours) * 3600 + StrToInt64(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(mSec - (((StrToInt64(tDays) * 86400) + StrToInt64(tHours) * 3600 + StrToInt64(tMinutes) * 60) + StrToInt64(tSecs)) * 1000);
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    tmSecs := Copy(tmSecs, 1, 2);
    if tDays <> '0' then begin
        if tDays = '1' then begin
            Result := tDays + ' day ' + tHours + ':' + tMinutes + ':' + tSecs{ + '.' + tmSecs};
        end else begin
            Result := tDays + ' days ' + tHours + ':' + tMinutes + ':' + tSecs{ + '.' + tmSecs};
        end;
    end else begin
        Result := tHours + ':' + tMinutes + ':' + tSecs{ + '.' + tmSecs};
    end;
end;

function mSec2TimeEx(mSec: Int64; Format: Integer): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  tmSecs: String;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    if mSec = High(Int64)
        then mSec := 0;
    Seconds := mSec div 1000;
    tHours := IntToStr(Seconds div 3600);
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    tMinutes := IntToStr(((Seconds - (StrToInt(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    tmSecs := IntToStr(mSec - ((StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60) + StrToInt(tSecs)) * 1000);
    case Length(tmSecs) of
        1: tmSecs := '00' + tmSecs;
        2: tmSecs := '0' + tmSecs;
    end;
    tmSecs := Copy(tmSecs, 1, 2);
    case Format of
        0: Result := tHours + ':' + tMinutes + ':' + tSecs;
        1: Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
        2: Result := tMinutes + ':' + tSecs;
        3: Result := tMinutes + ':' + tSecs + '.' + tmSecs;
        4: Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + Copy(tmSecs, 1, 1);
    else Result := tHours + ':' + tMinutes + ':' + tSecs + '.' + tmSecs;
    end;
end;

function mSec2HoursPart(mSec: Int64): Int64;
var
  Hours: Int64;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    Hours := Seconds div 3600;
    Result := Hours;
end;

function mSec2MinutesPart(mSec: Int64): Int64;
var
  Hours: Int64;
  Minutes: Int64;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    Hours := Seconds div 3600;
    Minutes := (Seconds - (Hours * 3600)) div 60;
    Result := Minutes;
end;

function mSec2SecondsPart(mSec: Int64): Int64;
var
  Hours: Int64;
  Minutes: Int64;
  Secs: Int64;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    Hours := Seconds div 3600;
    Minutes := (Seconds - (Hours * 3600)) div 60;
    Secs := Seconds - (Hours * 3600 + Minutes * 60);
    Result := Secs;
end;

function mSec2MilliSecondsPart(mSec: Int64): Int64;
var
  Hours: Int64;
  Minutes: Int64;
  Secs: Int64;
  mSecs: Int64;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    Hours := Seconds div 3600;
    Minutes := (Seconds - (Hours * 3600)) div 60;
    Secs := Seconds - (Hours * 3600 + Minutes * 60);
    mSecs := mSec - ((Hours * 3600 + Minutes * 60) + Secs) * 1000;
    Result := mSecs;
end;

function mSec2TimeSecs(mSec: Int64): String;
var
  tHours: String;
  tMinutes: String;
  tSecs: String;
  Seconds: Int64;
begin
    if mSec = 4294967295
        then mSec := 0;
    Seconds := mSec div 1000;
    tHours := IntToStr(Seconds div 3600);
    {
    if Length(tHours) = 1
        then tHours := '0' + tHours;
    }
    tMinutes := IntToStr(((Seconds - (StrToInt(tHours) * 3600)) div 60));
    if Length(tMinutes) = 1
        then tMinutes := '0' + tMinutes;
    tSecs := IntToStr(Seconds - (StrToInt(tHours) * 3600 + StrToInt(tMinutes) * 60));
    if Length(tSecs) = 1
        then tSecs := '0' + tSecs;
    Result := tHours + ':' + tMinutes + ':' + tSecs;
end;

function NGetFileSize(FileName: String): Int64;
var
  f: TSearchRec;
begin
    Result := - 1;
    if FileExists(FileName) then begin
    	try
            if FindFirst(FileName, faAnyFile, f) = 0 then begin
                //Result := MakeInt64(f.FindData.nFileSizeLow, f.FindData.nFileSizeHigh);
                Result := f.Size;
            end;
        finally
            SysUtils.FindClose(f);
        end;
    end;
end;

function NGetFileSizeFrom(SearchRec: TSearchRec): Int64;
begin
    Result := - 1;
    if SearchRec.Name <> '' then begin
    	try
            //if SearchRec.FindData.nFileSizeHigh > 0 then begin
                //Result := MakeInt64(SearchRec.FindData.nFileSizeLow, SearchRec.FindData.nFileSizeHigh);
                Result := SearchRec.Size;
            //end;
        except
          	//*
        end;
    end;
end;

function EncodeUnicode(UniString: String): String;
begin
    Result := StringReplace(UniString, '&', '&amp;', [rfReplaceAll]);
    Result := StringReplace(Result, '<', '&lt;', [rfReplaceAll]);
    Result := StringReplace(Result, '>', '&gt;', [rfReplaceAll]);
    Result := StringReplace(Result, '"', '&quot;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#162;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#163;', [rfReplaceAll]);
    Result := StringReplace(Result, '|', '&#166;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#167;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&copy;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#171;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#187;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&reg;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#177;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#180;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#181;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#164;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#165;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#182;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#183;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#189;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Aacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Atilde;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Auml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&AElig;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ccedil;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Eacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ecirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Euml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Iacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Icirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ETH;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ntilde;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ograve;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Oacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ocirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Otilde;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ouml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#215;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Oslash;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ugrave;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Uacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Ucirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Uuml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Uuml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&Yacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&szlig;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&aacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&acirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&atilde;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&auml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&aring;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&aelig;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ccedil;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&egrave;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&eacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ecirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&euml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&igrave;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&iacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&icirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&iuml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&icirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&eth;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ntilde;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ograve;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&oacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ocirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&otilde;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ouml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&#247;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&oslash;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ugrave;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&uacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&ucirc;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&uuml;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&yacute;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&thorn;', [rfReplaceAll]);
    Result := StringReplace(Result, '', '&yuml;', [rfReplaceAll]);
end;

function DecodeUnicode(UnicodeString: String): String;
begin
    Result := StringReplace(UnicodeString, '&amp;', '&', [rfReplaceAll]);
    Result := StringReplace(Result, '&#32;', ' ', [rfReplaceAll]);
    Result := StringReplace(Result, '&lt;', '<', [rfReplaceAll]);
    Result := StringReplace(Result, '&gt;', '>', [rfReplaceAll]);
    Result := StringReplace(Result, '&quot;', '"', [rfReplaceAll]);
    Result := StringReplace(Result, '&nbsp;', ' ', [rfReplaceAll]);
    Result := StringReplace(Result, '&#162;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#163;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#166;', '|', [rfReplaceAll]);
    Result := StringReplace(Result, '&#167;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&copy;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#171;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#187;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&reg;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#177;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#180;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#181;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#164;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#165;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#182;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#183;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#189;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Aacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Atilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Auml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&AElig;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ccedil;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Eacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ecirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Euml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Iacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Icirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ETH;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ntilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ograve;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Oacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ocirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Otilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ouml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#215;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Oslash;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ugrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Uacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Ucirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Uuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Uuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&Yacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&szlig;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&aacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&acirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&atilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&auml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&aring;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&aelig;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ccedil;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&egrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&eacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ecirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&euml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&igrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&iacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&icirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&iuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&icirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&eth;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ntilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ograve;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&oacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ocirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&otilde;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ouml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&#247;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&oslash;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ugrave;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&uacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&ucirc;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&uuml;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&yacute;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&thorn;', '', [rfReplaceAll]);
    Result := StringReplace(Result, '&yuml;', '', [rfReplaceAll]);
end;

function IntToBin1(Value: Longint; Digits: Integer): string;
var
  i: Integer;
begin
  Result := '';
  for i := Digits - 1 downto 0 do
    if Value and (1 shl i) <> 0 then
      Result := Result + '1'
  else
    Result := Result + '0';
end;

function IntToBin2(d: Longint): string;
var
  x: Integer;
  bin: string;
begin
  bin := '';
  for x := 1 to 8 * SizeOf(d) do
  begin
    if Odd(d) then bin := '1' + bin
    else
      bin := '0' + bin;
    d := d shr 1;
  end;
  Delete(bin, 1, 8 * ((Pos('1', bin) - 1) div 8));
  Result := bin;
end;

function URLEncode(const S: String; const InQueryString: Boolean): String;
var
  Idx: Integer; // loops thru characters in string
begin
  Result := '';
  for Idx := 1 to Length(S) do
  begin
    case S[Idx] of
      'A'..'Z', 'a'..'z', '0'..'9', '-', '_', '.':
        Result := Result + S[Idx];
      ' ':
        if InQueryString then
          Result := Result + '+'
        else
          Result := Result + '%20';
      else
        Result := Result + '%' + SysUtils.IntToHex(Ord(S[Idx]), 2);
    end;
  end;
end;

function GetFileModDate(FileName: String): TDateTime;
var
    F: TSearchRec;
begin
    FindFirst(FileName, faAnyFile, F);
    //Result := F.Time;
    //if you wanted a TDateTime, change the return type and use this line:
    Result := F.TimeStamp;
    SysUtils.FindClose(F);
end;

function ExtractFileNameFromURL(const FileName: string): string;
var
    I: Integer;
begin
    I := LastDelimiter(PathDelim + DriveDelim + '/', FileName);
    Result := Copy(FileName, I + 1, MaxInt);
end;

function IncludeTrailingPathDelimiterHTTP(const S: string): string;
begin
  Result := S;
  if not IsDelimiter('/', Result, Length(Result)) then
    Result := Result + '/';
end;

function DateTimeDiff(Start, Stop: TDateTime): Int64;
var
    TimeStamp : TTimeStamp;
begin
    try
        TimeStamp := DateTimeToTimeStamp(Stop - Start);
        Dec(TimeStamp.Date, TTimeStamp(DateTimeToTimeStamp(0)).Date);
        Result := (TimeStamp.Date*24*60*60)+(TimeStamp.Time div 1000);
    except
        Result := 0;
    end;
end;

function DateTimeDiffmSec(Start, Stop: TDateTime): Int64;
var
    TimeStamp : TTimeStamp;
begin
    TimeStamp := DateTimeToTimeStamp(Stop - Start);
    Dec(TimeStamp.Date, TTimeStamp(DateTimeToTimeStamp(0)).Date);
    Result := (TimeStamp.Date*24*60*60*1000)+(TimeStamp.Time);
end;


function IntToStrDigits(Value: Int64; Digits: Integer): String;
var
    i: Integer;
begin
    Result := IntToStr(Value);
    if Length(Result) < Digits then begin
        for i := 1 to Digits - Length(Result) do begin
            Result := '0' + Result;
        end;
    end;
end;

function Time2mSec(Value: TTime): Int64;
Const
    MS = 1.15740740740741E-8;
begin
    Result := Round(Value / MS)
end;

function Linear2dB(Value: Double): String;
var
    dB: Double;
begin
    if Value = 0 then begin
        Result := '-inf dB'
    end else begin
        dB := log10(Value) * 20;
        if Abs(dB) = dB then begin
            if dB = 0 then begin
                Result := FloatToStrF(dB, ffFixed, 6, 2) + ' dB';
            end else begin
                Result := '+' + FloatToStrF(dB, ffFixed, 6, 2) + ' dB';
            end;
        end else begin
            Result := FloatToStrF(dB, ffFixed, 6, 2) + ' dB';
        end;
    end;
end;

function dB2LinearPercent(Value: Double): String;
begin
    Result := FloatToStrF(Power(10, (- Value) / 20) * 100, ffFixed, 6, 2) + ' %';
end;

function HTMLDecode(const AStr: String): String;
var
  Sp, Rp, Cp, Tp: PChar;
  S: String;
  I, Code: Integer;
begin
  SetLength(Result, Length(AStr));
  Sp := PChar(AStr);
  Rp := PChar(Result);
  //Cp := Sp;
  try
    while Sp^ <> #0 do
    begin
      case Sp^ of
        '&': begin
               //Cp := Sp;
               Inc(Sp);
               case Sp^ of
                 'a': if AnsiStrPos(Sp, 'amp;') = Sp then  { do not localize }
                      begin
                        Inc(Sp, 3);
                        Rp^ := '&';
                      end;
                 'l',
                 'g': if (AnsiStrPos(Sp, 'lt;') = Sp) or (AnsiStrPos(Sp, 'gt;') = Sp) then { do not localize }
                      begin
                        Cp := Sp;
                        Inc(Sp, 2);
                        while (Sp^ <> ';') and (Sp^ <> #0) do
                          Inc(Sp);
                        if Cp^ = 'l' then
                          Rp^ := '<'
                        else
                          Rp^ := '>';
                      end;
                 'n': if AnsiStrPos(Sp, 'nbsp;') = Sp then  { do not localize }
                      begin
                        Inc(Sp, 4);
                        Rp^ := ' ';
                      end;
                 'q': if AnsiStrPos(Sp, 'quot;') = Sp then  { do not localize }
                      begin
                        Inc(Sp,4);
                        Rp^ := '"';
                      end;
                 '#': begin
                        Tp := Sp;
                        Inc(Tp);
                        while (Sp^ <> ';') and (Sp^ <> #0) do
                          Inc(Sp);
                        SetString(S, Tp, Sp - Tp);
                        Val(S, I, Code);
                        Rp^ := Chr((I));
                      end;
                 else
                   Exit;
               end;
           end
      else
        Rp^ := Sp^;
      end;
      Inc(Rp);
      Inc(Sp);
    end;
  except
  end;
  SetLength(Result, Rp - PChar(Result));
end;

function ReplaceNewLines(Text: String): String;
Const
    TEMP_NEWLINE = '{NL}';
var
    TempText: String;
begin
    TempText := StringReplace(Text, #13#10, TEMP_NEWLINE, [rfReplaceAll]);
    TempText := StringReplace(TempText, #10#13, TEMP_NEWLINE, [rfReplaceAll]);
    TempText := StringReplace(TempText, #13, TEMP_NEWLINE, [rfReplaceAll]);
    Result := StringReplace(TempText, TEMP_NEWLINE, #13#10, [rfReplaceAll]);
end;

function FFTIndex2Frequency(Index: Integer; Length: Integer; Samplerate: Integer): Integer;
begin
    Result := Integer(Round((Index * Samplerate) / Length));
end;

function FFTFrequency2Index(Frequency: Integer; Length: Integer; Samplerate: Integer): Integer;
var
    i: Integer;
begin
    i := Integer(Round((Length * Frequency) / Samplerate));
    if (i > ((Length / 2) - 1)) then begin
        i := Round((Length / 2) - 1);
    end;
    Result := i;
end;

function Min(const B1, B2: Integer): Integer;
begin
    if B1 < B2 then begin
        Result := B1
    end else begin
        Result := B2;
    end;
end;

function Max(const B1, B2: Integer): Integer;
begin
    if B1 > B2 then begin
        Result := B1
    end else begin
        Result := B2;
    end;
end;

function GoUpOneFolder(Folder: String): String;
begin
    Result := Folder;
    if Length(Result) > 1 then begin
        Result := ExtractFilePath(Copy(Result, 1, Length(Result) - 2));
    end;
    if NOT DirectoryExists(Result) then begin
        Result := '';
    end;
end;

function GoUpOneCastFolder(Folder: String): String;
begin
    Result := Folder;
    if Length(Result) > 0 then begin
        Result := ExtractFilePath(Copy(Result, 1, Length(Result) - 2));
    end;
end;

function ExtractSeasonEpisode(Text: String; var S: Integer; var E: Integer): Boolean;
var
    Offset: Integer;
    SPos, EPos: Integer;
    SStr, EStr: String;
    EEnd: Char;
    Found: Boolean;
begin
    Result := False;
    S := 0;
    E := 0;
    Text := UpperCase(Text);
    Offset := 1;
    Found := False;
    repeat
        SPos := Pos('S', Text, Offset);
        EPos := Pos('E', Text, SPos);
        if EPos - SPos < 5 then begin
            SStr := Copy(Text, SPos + 1, EPos - SPos - 1);
            EStr := Copy(Text, EPos + 1, 3);
            SStr := Trim(SStr);
            EStr := Trim(EStr);
            EEnd := EStr[Length(EStr)];
            if NOT EEnd.IsDigit then begin
                EStr := Copy(EStr, 1, Length(EStr) - 1);
            end;
            S := StrToIntDef(SStr, - 1);
            E := StrToIntDef(EStr, - 1);
            if (S > - 1)
            AND (E > - 1)
            then begin
                Found := True;
                Result := True;
            end;
        end;
        Offset := SPos + 1;
    until Found
    OR (SPos = 0)
    OR (Offset >= Length(Text));
end;

end.



