unit EmailAccountSettings;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics, UITypes,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Samples.Spin, IdSSLOpenSSL, QueryEmails;

type
  TFormEditAccount = class(TForm)
    Button1: TButton;
    Button2: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    EditName: TEdit;
    EditHost: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    SpinEditPort: TSpinEdit;
    LabelPortDefault: TLabel;
    EditLoginName: TEdit;
    Label5: TLabel;
    GroupBox2: TGroupBox;
    Label6: TLabel;
    ComboBoxSecurity: TComboBox;
    EditPassword: TEdit;
    Label7: TLabel;
    Button3: TButton;
    Label8: TLabel;
    ComboBoxSSLMethod: TComboBox;
    Label9: TLabel;
    ComboBoxProtocol: TComboBox;
    Label10: TLabel;
    procedure FormShow(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure ComboBoxProtocolChange(Sender: TObject);
    procedure ComboBoxSecurityChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    AccountItemData: TAccountItemData;
    AlreadyFilled: Boolean;
    procedure FillDataFromForm(AccountItem: TAccountItemData);
  end;

implementation

{$R *.dfm}

procedure TFormEditAccount.FillDataFromForm(AccountItem: TAccountItemData);
begin
    //* Fill
    AccountItem.ItemName := EditName.Text;
    AccountItem.HostName := EditHost.Text;
    AccountItem.Port := SpinEditPort.Value;
    AccountItem.LoginName := EditLoginName.Text;
    AccountItem.Password := EditPassword.Text;
    AccountItem.Security := TAccountSecurity(ComboBoxSecurity.ItemIndex);
    AccountItem.SSLMethod := TIdSSLVersion(ComboBoxSSLMethod.ItemIndex);
    AccountItem.ServerType := TServerAccountType(ComboBoxProtocol.ItemIndex);
end;

procedure TFormEditAccount.Button1Click(Sender: TObject);
begin
    FillDataFromForm(AccountItemData);
end;

procedure TFormEditAccount.Button3Click(Sender: TObject);
var
    TestAccountItem: TAccountItemData;
begin
    TestAccountItem := TAccountItemData.Create;
    try
        FillDataFromForm(TestAccountItem);
        case ComboBoxProtocol.ItemIndex of
            0: begin
                if TestConnectionPOP3(TestAccountItem) then begin
                    MessageDlg('Connection successful.', mtInformation, [mbOk], 0);
                end;
            end;
            1: begin
                if TestConnectionIMAP(TestAccountItem) then begin
                    MessageDlg('Connection successful.', mtInformation, [mbOk], 0);
                end;
            end;
        end;
    finally
        FreeAndNil(TestAccountItem);
    end;
end;

procedure TFormEditAccount.ComboBoxProtocolChange(Sender: TObject);
begin
    case ComboBoxProtocol.ItemIndex of
        0: LabelPortDefault.Caption := 'Default: 995';
        1: LabelPortDefault.Caption := 'Default: 993';
    end;
end;

procedure TFormEditAccount.ComboBoxSecurityChange(Sender: TObject);
begin
    case ComboBoxSecurity.ItemIndex of
        0: begin
            Label9.Enabled := False;
            ComboBoxSSLMethod.Enabled := False;
        end;
        1: begin
            Label9.Enabled := True;
            ComboBoxSSLMethod.Enabled := True;
        end;
    end;
end;

procedure TFormEditAccount.FormShow(Sender: TObject);
begin
    if AlreadyFilled then begin
        Exit;
    end;
    AlreadyFilled := True;
    //* Fill
    if AccountItemData.ItemName <> '' then begin
        EditName.Text := AccountItemData.ItemName;
    end;
    if AccountItemData.HostName <> '' then begin
        EditHost.Text := AccountItemData.HostName;
    end;
    if AccountItemData.Port <> 0 then begin
        SpinEditPort.Value := AccountItemData.Port;
    end;
    if AccountItemData.LoginName <> '' then begin
        EditLoginName.Text := AccountItemData.LoginName;
    end;
    if AccountItemData.Password <> '' then begin
        EditPassword.Text := AccountItemData.Password;
    end;
    ComboBoxSecurity.ItemIndex := Ord(AccountItemData.Security);
    ComboBoxSSLMethod.ItemIndex := Ord(AccountItemData.SSLMethod);
    ComboBoxProtocol.ItemIndex := Ord(AccountItemData.ServerType);
end;

end.
